import {
  Metadata, Loader, Http, IEventParam, IDictionary, Config, Datasource,
} from '@zeedhi/core';
import {
  GridEditable, Form, IButtonEvent, IGridEvent, ITabs, Select, Button,
  TextInput, Grid, DialogService, CollapseCard, Modal, ModalService,
  SelectMultiple, TreeGridEditable, IComponentRender, LoadingService,
} from '@zeedhi/common';
import { PmeCurrencyMask, PmeMask } from '@/config';
  
export class Pme00300TipoDeCardapioController {
  private utilities = Loader.getInstance('Utilities');
  
  private modalRelatorio!: Modal;
  
  public appController = Loader.getInstance('AppController');
  
  public modalController = Loader.getInstance('ModalController');

  public formTpCardapioController = Loader.getInstance('FormTpCardapioController');

  public gridEditController = Loader.getInstance('GridEditableController');
  
  public reportController = Loader.getInstance('ReportController');
  
  public crudController = Loader.getInstance('CrudController');

  public editable = false;
  
  public newRow = false;

  public activeArv: IDictionary = {};

  public selectedTipoCard?: string = '';

  public selectedServ?: string = '';

  public selectedNmServ?: string = '';
  
  public selectedUnidade?: string = '';

  public selectedNmUnidade?: string = '';

  public showConfig = false;
  
  public showCardConfig = false;

  public colsServ = 12;
  
  public buttonRelatorioTpCard = false;
  
  public buttonRelatorioCollapse = false;
  
  public reportUnidadeVisibility = false;
  
  public reportServiceVisibility = true;

  public get colsConfigServ() {
    return this.colsServ % 12;
  }

  public showTipoPrat = false;

  private maxNrSeq = 0;
  
  public filterTipoCardapio() {
    const cardapio = Metadata.getInstance<GridEditable>('TIPOCARD');
    const cdfilcentpme = this.appController.getCdCentral();
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: cdfilcentpme,
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: this.activeArv.NRPASTATCARD || null,
      },
    ];
    cardapio.datasource.filter = conditions;
    if (cardapio.editing) {
      this.gridEditController.defaultState(cardapio).then(() => {
        this.clearSelectedVal(false);
        this.filterGridServ();
        this.filterUnidadeTipoCard();
      });
    } else {
      cardapio.datasource.get().then(() => {
        this.clearSelectedVal(false);
        this.filterGridServ();
        this.filterUnidadeTipoCard();
      });
    }
  }

  public saveArvTpCard() {
    const form = Metadata.getInstance<Form>('formTpCardapio');
    if (form.validate()) {
      const row = form.value;
      const arvPai = this.formTpCardapioController.getArvPai();
      if (this.formTpCardapioController.semPai) {
        row.NRPASTAPAI = null;
        row.NMPASTAPAI = null;
        row.NIVELPAI = null;
      } else {
        row.NRPASTAPAI = arvPai.NRPASTATCARD ? arvPai.NRPASTATCARD : null;
        row.NMPASTAPAI = arvPai.NMPASTATCARD ? arvPai.NMPASTATCARD : null;
        row.NIVELPAI = arvPai.NRNVPASTCARD ? arvPai.NRNVPASTCARD : null;
      }
      row.CDFILCENTPME = this.appController.getCdCentral();
      LoadingService.show(' ');
      Http.post('/saveArvTpCard', { requestType: 'Row', row }).then((response) => {
        this.formTpCardapioController.semPai = false;
        if (response) {
          const nome = Metadata.getInstance<TextInput>('NMPASTATCARD');
          nome.value = '';

          const gridServico = Metadata.getInstance<GridEditable>('SERVTIPOCARD');
          const gridTpCard = Metadata.getInstance<GridEditable>('TIPOCARD');

          this.editable = false;
          
          gridServico.datasource.filter = [];
          gridTpCard.datasource.filter = [];

          if (gridServico.editing) {
            this.gridEditController.defaultState(gridServico).then(() => {
              gridServico.datasource.data = [];
            });
          } else gridServico.datasource.data = [];

          if (gridTpCard.editing) {
            this.gridEditController.defaultState(gridTpCard).then(() => {
              gridTpCard.datasource.data = [];
            });
          } else gridTpCard.datasource.data = [];

          this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'green');
          this.modalController.closeModal();
          this.reloadTree();
        }
      }).finally(() => {
        LoadingService.hide();
      });
    }
  }

  public saveTipoCard() {
    const form = Metadata.getInstance<Form>('formTipocard');
    const grid = Metadata.getInstance<GridEditable>('TIPOCARD');
    const rows: IDictionary = grid.getEditedRows();
    if (form.validate() && rows.length > 0) {
      rows.forEach((e: IDictionary) => {
        e.NRPASTATCARD = this.activeArv ? (this.activeArv as IDictionary).NRPASTATCARD : null;
        e.CDFILCENTPME = this.appController.getCdCentral();
      });
      this.callSave('/saveTipoCard', rows, grid);
    }
  }

  public saveServTpCard() {
    const form: Form = Metadata.getInstance<Form>('formServico');
    const grid = Metadata.getInstance<GridEditable>('SERVTIPOCARD');
    const rows: IDictionary = grid.getEditedRows();
    if (form.validate() && rows.length > 0) {
      const central = this.appController.getCdCentral();
      rows.forEach((e: IDictionary) => {
        // if (e.__is_new) e.CDSERVFILI = e.NMSERVICO;
        // e.CDPRODUTO = e.NMPRODUTO || null;
        e.CDTIPOCARD = this.selectedTipoCard;
        e.CDFILCENTPME = central;
      });
      if (central === null) this.utilities.showAlert('alertError', 'pme::centralNaoInformada', 'red');
      else this.callSave('/saveServTpCard', rows, grid);
    }
  }

  public saveCusto() {
    const form = Metadata.getInstance<Form>('formCustoPadr');
    const grid = Metadata.getInstance<GridEditable>('CUSTPADR');
    if (form.validate()) {
      const rows: IDictionary = grid.getEditedRows();
      if (rows.length > 0) {
        rows.forEach((e: IDictionary) => {
          e.CDFILCENTPME = this.appController.getCdCentral();
          e.CDSERVFILI = this.selectedServ;
          e.CDTIPOCARD = this.selectedTipoCard;
        });
  
        this.callSave('/saveCusto', rows, grid);
      } else {
        this.gridEditController.defaultState(grid);
      }
    }
  }
  
  public async deleteConf({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const checkedRows = grid.selectedRows;
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      await DialogService.show({
        name: component.name,
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yesDelete',
            component: 'ZdButton',
            label: 'pme::sim',
            events: {
              click: () => {
                LoadingService.show(' ');
                this.crudController.deleteRows(checkedRows, 'confTipoPrato').then(() => {
                  grid.cancelEditedRows();
                  // grid.datasource.get();
                  // this.filterGridConfPrat();
                  grid.selectedRows = [];
                  this.utilities.showAlert('alert', 'pme::excluidoComSucesso', 'success');
                }).catch((error: string) => {
                  this.utilities.showAlert('alert', `pme::${error}`, 'error');
                }).finally(() => {
                  LoadingService.hide();
                  DialogService.hide();
                });
              },
            },
            outline: true,
          },
          {
            name: 'noDelete',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }

  public async saveConfigPrat() {
    const form = Metadata.getInstance<Form>('formConfig');
    const grid = Metadata.getInstance<GridEditable>('CONFIGTIPOPRAT');
    // const tipoPrat = Metadata.getInstance<Select>('TIPOPRAT');
    const cdfilcentpme = this.appController.getCdCentral();

    const rows: IDictionary = grid.getEditedRows();
    
    // let maxNrSeq = grid.datasource.totalValue;
    
    if (form.validate() && rows.length > 0) {
      const newRows: IDictionary[] = [];
      await rows.forEach((row: IDictionary) => {
        if (row.NRORDCNFTCAR === undefined && row.NRSEQCNFTCAR === undefined) {
          row.CDTIPOPRAT.forEach((e: IDictionary) => {
            // maxNrSeq += 1;
            const newRow = {
              CDTIPOPRAT: e.CDTIPOPRAT,
              NMTIPOPRAT: e.NMTIPOPRAT,
              // NRORDCNFTCAR: maxNrSeq,
              // NRSEQCNFTCAR: maxNrSeq,
              IDIMPCNFTCAR: row.IDIMPCNFTCAR || 'N',
              IDOPCCNFTCAR: row.IDOPCCNFTCAR || 'N',
              NEW: 'S',
              CDFILCENTPME: cdfilcentpme,
              CDSERVFILI: this.selectedServ,
              CDTIPOCARD: this.selectedTipoCard,
            };
            newRows.push(newRow);
          });
        } else {
          if (Array.isArray(row.CDTIPOPRAT)) row.CDTIPOPRAT = row.CDTIPOPRAT[0];
          if (Array.isArray(row.NMTIPOPRAT)) row.NMTIPOPRAT = row.NMTIPOPRAT[0];
          row.NEW = 'N';
          newRows.push(row);
        }
      });
      
      this.callSave('/saveConfigPrat', newRows, grid);
    }
  }

  private callSave(url: string, rows: IDictionary, grid: GridEditable) {
    if (grid) LoadingService.show(' ');
    Http.post(url, { requestType: 'Row', row: rows }).then((response) => {
      if (response) {
        this.editable = false;
        this.utilities.showAlert('alertSuccess', 'pme::salvoComSucesso', 'green');
        
        if (url.includes('saveServTpCard') && rows[0].__is_new) {
          this.selectedServ = rows[0].CDSERVFILI;
          this.realizaAcaoExportacao(false);
        }
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
    }).finally(() => {
      if (grid.name === 'CONFIGTIPOPRAT') {
        this.gridEditController.defaultState(grid);
        grid.selectedRows = [];
      }
      if (grid) LoadingService.hide();
    });
  }

  public addArvTipoCard() {
    const treeGrid = Metadata.getInstance<TreeGridEditable>('TREEGRID');
    const verificaEdit = treeGrid.editing;
    if (this.utilities.centralIsValid() && !verificaEdit) {
      this.modalController.openModal();
      this.formTpCardapioController.setArvPai(this.activeArv);
      if (!this.activeArv.NRPASTATCARD) this.formTpCardapioController.semPai = true;
    }
  }
  
  public closeArvTipoCard() {
    this.modalController.closeModal();
  }

  public addRow({ component }: IDictionary) {
    if (this.utilities.centralIsValid() && !this.gridEditController.isInlineEditing) {
      this.editable = true;
      let grid: GridEditable;
      
      if (component.type === 'button') grid = component.parent as GridEditable;
      else grid = component;

      if (!this.activeArv.NRPASTATCARD) this.utilities.showAlert('alertError', 'pme::tipoArvCardNaoSelecionada', 'warning');
      else if (grid.name !== 'TIPOCARD' && !this.selectedTipoCard) {
        this.utilities.showAlert('alertError', 'pme::tipocardNaoSelecionado', 'warning');
      } else if (!this.gridEditController.newRow) {
        const item = { IDTPCARDATIV: '', __is_new: true };
        if (grid.name === 'TIPOCARD') {
          item.IDTPCARDATIV = 'S';
          this.clearSelectedVal(true);
          const gridServico = Metadata.getInstance<GridEditable>('SERVTIPOCARD');
          const tabPai = Metadata.getInstance<ITabs>('tabPai');
          gridServico.datasource.data = [];
          
          if (tabPai.activeTab === 1) {
            const gridUnidadeTpCard = Metadata.getInstance<GridEditable>('UNIDADECARD');
            gridUnidadeTpCard.datasource.data = [];
          }
        }
        this.gridEditController.addRow({ component: grid });
        // grid.datasource.data.unshift(item);
        // this.gridEditController.newRow = true;
        // grid.editing = true;
        if (!this.gridEditController.isInlineEditing) this.gridEditController.onInlineEdit({ component: grid });
      }
    }
  }
  
  public addConf({ component }: IDictionary) {
    if (this.utilities.centralIsValid() && !this.gridEditController.isInlineEditing) {
      this.editable = true;
      let grid: GridEditable;
      
      if (component.type === 'button') grid = component.parent as GridEditable;
      else grid = component;

      if (!this.activeArv.NRPASTATCARD) this.utilities.showAlert('alertError', 'pme::tipoArvCardNaoSelecionada', 'warning');
      else if (grid.name !== 'TIPOCARD' && !this.selectedTipoCard) {
        this.utilities.showAlert('alertError', 'pme::tipocardNaoSelecionado', 'warning');
      } else if (!this.gridEditController.newRow) {
        // this.showTipoPrat = true;
        // this.gridEditController.newRow = true;
        // grid.editing = true;
        // if (!this.gridEditController.isInlineEditing) this.gridEditController.onInlineEdit({ component: grid });
        this.gridEditController.addRowSetFocusSecondPosition({ component });
      }
    }
  }

  public cancelEditing({ component }: IButtonEvent | IGridEvent) {
    this.editable = false;
    this.showTipoPrat = false;
    const tipoPrat = Metadata.getInstance<Select>('TIPOPRAT');
    tipoPrat.setValue([]);
    this.gridEditController.cancelEditing({ component });
  }

  public cancelEditingGrid({ component }: IButtonEvent | IGridEvent) {
    this.editable = false;
    const grid: GridEditable = this.utilities.returnGrid(component) as GridEditable;
    if (grid.editing) {
      this.gridEditController.defaultState(grid).then(() => {
        const firstRow = grid.datasource.data[0];
        grid.datasource.currentRow = firstRow || null;
        if (grid.name === 'TIPOCARD') {
          this.selectedTipoCard = firstRow?.CDTIPOCARD || '';
          this.clearSelectedVal(false);
          this.filterGridServ();
          this.filterUnidadeTipoCard();
        } else if (grid.name === 'SERVTIPOCARD') {
          this.selectedServ = firstRow?.CDSERVFILI;
          this.selectedNmServ = `${firstRow?.CDSERVFILI} - ${firstRow?.NMSERVICO}`;
        }
      }).catch((error: string) => {
        this.utilities.showAlert('alertError', `pme::${error}`, 'error');
      });
    }
  }

  public setActiveArvTpCard() {
    const gridTipoCard = Metadata.getInstance<GridEditable>('TIPOCARD');
    const treeGrid = Metadata.getInstance<TreeGridEditable>('TREEGRID');
    const row = treeGrid.datasource.currentRow;
    gridTipoCard.selectedRows = [];

    this.activeArv = row;
    this.formTpCardapioController.semPai = false;
    if (row.CDFILCENTPME) this.filterTipoCardapio();
  }

  public setTipoCard({ component }: IEventParam<Datasource>) {
    const gridTipoCard = Metadata.getInstance<GridEditable>('TIPOCARD');
    const verificaEdit = gridTipoCard.editing;
    const row = component.currentRow;
    this.selectedTipoCard = row.CDTIPOCARD;
    this.clearSelectedVal(false);
    if (!verificaEdit) {
      this.filterGridServ();
      this.filterUnidadeTipoCard();
    }
    
    this.showCardConfig = !!this.selectedTipoCard;
  }

  private clearSelectedVal(clearTpCard = false) {
    const tabs = Metadata.getInstance<ITabs>('tabsConfigServ');
    tabs.activeTab = 0;
    if (clearTpCard) this.selectedTipoCard = '';
    this.selectedServ = '';
    this.selectedNmServ = '';
  }

  public setServ({ component }: IEventParam<Datasource>) {
    const row = component.currentRow;
    this.selectedServ = row.CDSERVFILI;
    this.selectedNmServ = `${row.CDSERVFILI} - ${row.NMSERVICO}`;
    if (row.CDSERVFILI) {
      const gridServ = Metadata.getInstance<GridEditable>('SERVTIPOCARD');
      if (!gridServ.editing) {
        this.showConfig = true;
        this.colsServ = 6;
        const tabs = Metadata.getInstance<ITabs>('tabsConfigServ');
        this.filterGrids({ component: tabs });
      }
    } else {
      this.showConfig = false;
      this.colsServ = 12;
    }
  }

  public filterGrids({ component }: IDictionary) {
    const gridServ = Metadata.getInstance<GridEditable>('SERVTIPOCARD');
    this.disableButtonsGrid(gridServ);
    if (component.name === 'tabPai') {
      if (component.activeTab === 1) this.filterUnidadeTipoCard();
    } else {
      if (component.activeTab === 0) this.filterGridConfPrat();
      if (component.activeTab === 1) this.filterGridCust();
    }
  }
  
  private disableButtonsGrid(grid: IDictionary) {
    if (grid.editing) {
      this.gridEditController.newRow = false;
      // grid.cancelEditedRows();
      // this.gridEditController.onInlineEdit({ component: grid });
      this.gridEditController.cancelEditedRows({ component: grid });
      this.filterGridServ();
    }
  }

  public filterGridServ() {
    const gridServico = Metadata.getInstance<GridEditable>('SERVTIPOCARD');
    const cdfilcentpme = this.appController.getCdCentral();
    if (this.selectedTipoCard) {
      const conditions = [
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: cdfilcentpme,
        },
        {
          name: 'CDTIPOCARD',
          operator: '=',
          value: this.selectedTipoCard,
        },
      ];
      
      gridServico.datasource.filter = conditions;
      if (gridServico.editing) this.gridEditController.defaultState(gridServico);
      else gridServico.datasource.get();
    } else {
      gridServico.datasource.filter = [];
      gridServico.datasource.data = [];
    }
  }

  public async filterGridConfPrat() {
    const gridConf = Metadata.getInstance<GridEditable>('CONFIGTIPOPRAT');
    if (this.selectedServ !== '' && this.selectedTipoCard !== '') {
      const cdfilcentpme = this.appController.getCdCentral();
      const conditions = [
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: cdfilcentpme,
        },
        {
          name: 'CDSERVFILI',
          operator: '=',
          value: this.selectedServ,
        },
        {
          name: 'CDTIPOCARD',
          operator: '=',
          value: this.selectedTipoCard,
        },
      ];
      
      // const tipoPrat = Metadata.getInstance<Select>('TIPOPRAT');
      // tipoPrat.setValue([]);
      this.showTipoPrat = false;
  
      gridConf.datasource.filter = conditions;
      gridConf.datasource.loading = true;
      if (gridConf.editing) await this.gridEditController.defaultState(gridConf);
      else gridConf.datasource.get();
    } else {
      gridConf.datasource.filter = [];
      gridConf.datasource.data = [];
    }
  }
  
  public changePagination({ component }: IEventParam<Datasource>) {
    component.loading = true;
    setTimeout(() => {
      this.formatData({ component });
    }, 200);
  }
  
  public async formatData({ component }: IEventParam<Datasource>) {
    if (component.data.length > 0) {
      const data = [...component.data];
      await data.forEach((el) => {
        if (!(Array.isArray(el.NMTIPOPRAT))) el.NMTIPOPRAT = [el.NMTIPOPRAT];
        if (!(Array.isArray(el.CDTIPOPRAT))) el.CDTIPOPRAT = [el.CDTIPOPRAT];
      });
      component.data = data;
      component.loading = false;
    } else component.loading = false;
  }
  
  public filterGridCust() {
    if (document.getElementsByName('CUSTPADR').length > 0) {
      const gridCust = Metadata.getInstance<GridEditable>('CUSTPADR');
      const cdfilcentpme = this.appController.getCdCentral();
      const conditions = [
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: cdfilcentpme,
        },
        {
          name: 'CDTIPOCARD',
          operator: '=',
          value: this.selectedTipoCard,
        },
        {
          name: 'CDSERVFILI',
          operator: '=',
          value: this.selectedServ,
        },
      ];
      gridCust.datasource.filter = conditions;
      if (gridCust.editing) this.gridEditController.defaultState(gridCust);
      else gridCust.datasource.get();
    }
  }

  public filterProd({ component }: IEventParam<Select>) {
    component.datasource.get();
  }
  
  public filterServ() {
    const cdfilcentpme = this.appController.getCdCentral();

    return [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdfilcentpme,
      },
      {
        name: 'IDTIPOSERV',
        operator: '=',
        value: 'M',
      },
    ];
  }

  public deleteServTpCard({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const checkedRows = grid.selectedRows;

    if (checkedRows.length === 0) this.utilities.showAlert('alertWarning', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: component.name,
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.callDeleteServ() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
          {
            name: 'progressLine',
            component: 'ZdProgress',
            label: 'Progress',
            indeterminate: true,
            cssClass: 'loading-progress-line',
          },
        ],
      });
    }
  }

  private callDeleteServ() {
    const grid = Metadata.getInstance<GridEditable>('SERVTIPOCARD');
    const checkedRows = grid.selectedRows;
    // const buttonYes = Metadata.getInstance<Button>('yes');
    checkedRows.forEach((e: IDictionary) => {
      e.CDTIPOCARD = this.selectedTipoCard;
    });
    LoadingService.show(' ');
    Http.post('/crud/delete', { checkedRows, dataSourceName: 'servTipoCard' }).then(() => {
      grid.cancelEditedRows();
      grid.datasource.get().then(() => {
        this.setSelectedServ();
        this.colsServ = 12;
      });
      grid.selectedRows = [];
      this.utilities.showAlert('alertSuccess', 'pme::excluidoComSucesso', 'green');
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
    }).finally(() => {
      LoadingService.hide();
      DialogService.hide();
    });
  }

  public get enableTab(): boolean {
    const gridTipoCard = Metadata.getInstance<GridEditable>('TIPOCARD');
    return (this.selectedServ === '' || this.selectedServ === undefined
            || gridTipoCard.editing || this.selectedServ === null);
  }
  
  public get enableTabServico(): boolean {
    const gridTipoCard = Metadata.getInstance<GridEditable>('TIPOCARD');
    return (gridTipoCard.editing);
  }

  public createModal() {
    this.modalController.createModal('pme/parametrizacao/modals/formTpCardapio');
  }

  public destroyModal() {
    this.modalController.destroyModal();
  }

  public setSelectedServ() {
    const gridServico = Metadata.getInstance<GridEditable>('SERVTIPOCARD');
    const firstRow = gridServico.datasource.data[0];
    if (firstRow) {
      this.selectedServ = firstRow.CDSERVFILI;
      this.selectedNmServ = `${firstRow.CDSERVFILI} - ${firstRow.NMSERVICO}`;
      this.colsServ = 6;
    } else {
      this.selectedServ = '';
      this.selectedNmServ = '';
      this.colsServ = 12;
    }
  }

  public setSelectedTpCard() {
    const gridTpCard = Metadata.getInstance<GridEditable>('TIPOCARD');
    const firstRow = gridTpCard.datasource.data[0];
    if (firstRow && firstRow.CDTIPOCARD) {
      this.selectedTipoCard = firstRow.CDTIPOCARD;
    } else {
      this.selectedTipoCard = '';
    }
  }
  
  public setSelectedUnidade() {
    const gridUnidade = Metadata.getInstance<GridEditable>('UNIDADECARD');
    const firstRow = gridUnidade.datasource.data[0];
    if (firstRow) {
      this.selectedUnidade = firstRow.CDFILIAL;
      this.selectedNmUnidade = `${firstRow.CDFILIAL} - ${firstRow.NMFILIAL}`;
    } else {
      this.selectedUnidade = '';
      this.selectedNmUnidade = '';
    }
  }

  public filterUnidadeTipoCard() {
    const tabPai = Metadata.getInstance<ITabs>('tabPai');
    if (tabPai.activeTab === 1) {
      setTimeout(() => {
        const gridUnidadeTpCard = Metadata.getInstance<GridEditable>('UNIDADECARD');
        const cdfilcentpme = this.appController.getCdCentral();
        const conditions = [
          {
            name: 'CDFILCENTPME',
            operator: '=',
            value: cdfilcentpme,
          },
          {
            name: 'CDTIPOCARD',
            operator: '=',
            value: this.selectedTipoCard,
          },
        ];
        
        gridUnidadeTpCard.datasource.filter = conditions;
        if (gridUnidadeTpCard.editing) {
          this.gridEditController.defaultState(gridUnidadeTpCard).then(() => {
            const firstRow = gridUnidadeTpCard.datasource.data[0];
            if (firstRow) {
              gridUnidadeTpCard.datasource.currentRow = firstRow;
            }
          });
        } else {
          gridUnidadeTpCard.datasource.get().then(() => {
            const firstRow = gridUnidadeTpCard.datasource.data[0];
            if (firstRow) {
              gridUnidadeTpCard.datasource.currentRow = firstRow;
            }
          });
        }
      }, 1);
    }
  }
  
  public addRowUnidadeTipoCard({ component }: IDictionary) {
    if (this.utilities.centralIsValid()) {
      const grid = this.utilities.returnGrid(component) as GridEditable;
      if (grid.name !== 'TIPOCARD' && !this.selectedTipoCard) {
        this.utilities.showAlert('alertError', 'pme::tipocardNaoSelecionado', 'warning');
      } else if (!this.gridEditController.newRow) {
        // const item = { IDTCARATFILI: 'S', __is_new: true };
        // grid.datasource.data.unshift(item);
        // this.gridEditController.newRow = true;
        // grid.editing = true;
        this.gridEditController.addRow({ component: grid });
        if (!this.gridEditController.isInlineEditing) this.gridEditController.onInlineEdit({ component: grid });
      }
    }
  }
  
  public saveUnidadeTipoCard({ component }: IDictionary) {
    const form = Metadata.getInstance<Form>('formUnidadeTpCard');
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const rows: IDictionary = grid.getEditedRows();

    if (form.validate() && rows.length > 0) {
      rows.forEach((e: IDictionary) => {
        if (e.__is_new) {
          e.CDFILCENTPME = this.appController.getCdCentral();
          e.CDTIPOCARD = this.selectedTipoCard;
          e.CDFILIAL = e.CDFILIAL.CDFILIAL;
          e.IDTCARATFILI = e.IDTCARATFILI ? e.IDTCARATFILI : 'S';
        }
      });
      this.callSave('/saveUnidadeTipoCard', rows, grid);
    }
  }
  
  public setFilterGridUnidade() {
    const grid = Metadata.getInstance<GridEditable>('gridTipoCardUnidade');
    const unidade = Metadata.getInstance<Select>('CDFILIAL');
    const filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidade.value,
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];

    grid.datasource.filter = filter;
    if (grid.editing) this.gridEditController.defaultState(grid);
    else grid.datasource.get();
  }

  public setFilterFieldUnidade() {
    return [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
  }

  public setFilterUnidade({ component }: IEventParam<Select>) {
    const filter = this.setFilterFieldUnidade();
    
    component.datasource.filter = filter;
    component.datasource.get();
  }

  public setFilterTpCardapio() {
    return [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
  }
  
  public addRowTipoCardUnidade({ component }: IDictionary) {
    const form = Metadata.getInstance<Form>('formFilter');
    const unidade = Metadata.getInstance<Select>('CDFILIAL');
    const grid = this.utilities.returnGrid(component) as GridEditable;

    if (this.utilities.centralIsValid() && form.validate()) {
      if (!this.newRow) {
        if (unidade.value !== null) {
          this.gridEditController.addRow({ component: grid });
          if (!this.gridEditController.isInlineEditing) this.gridEditController.onInlineEdit({ component: grid });
        }
      } else unidade.validations.required.message = 'pme::campoObrigatorio';
    }
  }
  
  public saveTipoCardUnidade({ component }: IDictionary) {
    const form = Metadata.getInstance<Form>('formTipoCardUn');
    const unidade = Metadata.getInstance<Select>('CDFILIAL');
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const rows: IDictionary = grid.getEditedRows();

    if (form.validate() && rows.length > 0) {
      rows.forEach((e: IDictionary) => {
        if (e.__is_new) {
          e.CDFILCENTPME = this.appController.getCdCentral();
          e.CDTIPOCARD = e.CDTIPOCARD.CDTIPOCARD;
          e.IDTCARATFILI = e.IDTCARATFILI ? e.IDTCARATFILI : 'S';
          e.IDFATUPROD = e.IDFATUPROD ? e.IDFATUPROD : 'N';
          e.CDFILIAL = unidade.value;
        }
      });
      this.callSave('/saveTipoCardUnidade', rows, grid);
    }
  }
  
  public setFilterTreeGrid() {
    const treeGrid = Metadata.getInstance<TreeGridEditable>('TREEGRID');
    const cdfilcentpme = this.appController.getCdCentral();
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: cdfilcentpme,
      },
      {
        name: 'NRPASTCARANT',
        operator: '=',
        value: 'T',
      },
    ];
    treeGrid.datasource.filter = filter;
    treeGrid.datasource.filter.limit = 1000;
    treeGrid.datasource.get();
  }
  
  public loadGridTipo({ component }: IEventParam<Grid>) {
    component.datasource.get().then(() => {
      const data = component.datasource.data;
      if (data.length > 0) component.datasource.currentRow = data[0];
    });
  }

  public reloadTree() {
    const treeGrid = Metadata.getInstance<TreeGridEditable>('TREEGRID');
    treeGrid.buildTree();
    this.activeArv = {};
    const cardapio = Metadata.getInstance<GridEditable>('TIPOCARD');
    const gridServico = Metadata.getInstance<GridEditable>('SERVTIPOCARD');

    cardapio.datasource.data = [];
    gridServico.datasource.data = [];

    this.clearSelectedVal(true);
    this.filterUnidadeTipoCard();
    if (treeGrid.editing) this.cancelEditingTreeGrid();
  }
  
  public servicoComponentProps = {
    component: 'ZdSelect',
    dataText: ['CDSERVFILI', 'NMSERVICO'],
    formatterDataText: ['CDSERVFILI', 'NMSERVICO'],
    dataValueOut: [
      {
        column: 'CDPRODUTO',
        columnOnGrid: 'CDPRODUTO',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
    ],
    dataValue: 'CDSERVFILI',
    dense: true,
    datasource: {
      uniqueKey: 'CDSERVFILI',
      route: '/find/servicoPorUnidade',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };
  
  public produtoComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValueOut: [
      {
        column: 'CDARVPROD',
        columnOnGrid: 'CDARVPROD',
      },
      {
        column: 'NMPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
    ],
    dataValue: 'CDPRODUTO',
    dense: true,
    datasource: {
      uniqueKey: 'CDPRODUTO',
      route: '/find/prodUltimoNivel',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    events: {
      click: this.filterProd,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };

  public deleteTree({ component }: IDictionary) {
    const treeGrid = Metadata.getInstance<TreeGridEditable>('TREEGRID');
    const row = treeGrid.datasource.currentRow;
    const verificaEdit = treeGrid.editing;
    if (!row.NRPASTATCARD || verificaEdit) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: component.name,
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluir',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.callDeleteTree() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }

  private callDeleteTree() {
    const treeGrid = Metadata.getInstance<TreeGridEditable>('TREEGRID');
    const row = treeGrid.datasource.currentRow;
    const rowDelete = {
      CDFILCENTPME: row.CDFILCENTPME,
      NRPASTATCARD: row.NRPASTATCARD,
    };
    LoadingService.show(' ');
    Http.post('/deleteTree', { requestType: 'Row', row: rowDelete }).then((response) => {
      if (response) {
        const gridServico = Metadata.getInstance<GridEditable>('SERVTIPOCARD');
        const gridTpCard = Metadata.getInstance<GridEditable>('TIPOCARD');

        this.editable = false;
        if (gridServico.editing) this.gridEditController.defaultState(gridServico);
        if (gridTpCard.editing) this.gridEditController.defaultState(gridTpCard);
        this.reloadTree();
        this.utilities.showAlert('alert', 'pme::excluidoComSucesso', 'success');
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
    }).finally(() => {
      LoadingService.hide();
      DialogService.hide();
    });
  }
  
  private realizaImportacaoConfig(cdSelectedServ: string | undefined) {
    const gridConf = Metadata.getInstance<GridEditable>('CONFIGTIPOPRAT');
    const gridTpCard = Metadata.getInstance<GridEditable>('TIPOCARD');
    const tpCard = gridTpCard.datasource.currentRow.CDTIPOCARD;
    const rowsGridConf = gridConf.datasource.data;
    const cdfilial = this.appController.getCdCentral();
    const parametrosDaQueryInsert = {
      CDFILIAL: cdfilial,
      CDSERVFILI: cdSelectedServ,
      CDTIPOCARD: tpCard,
    };
    Http.post('/importaConfigPadraoTpCard', { rowsGridConf, parametrosDaQueryInsert }).then(() => {
      DialogService.hide();
      this.utilities.showAlert('alert', 'pme::importadoComSucesso', 'success');
      gridConf.datasource.get();
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
      DialogService.hide();
    });
  }
  
  public realizaAcaoExportacao(showMessage = true) {
    const cdfilial = this.appController.getCdCentral();
    const parametrosDaQuery = {
      CDFILIAL: cdfilial,
      CDSERVFILI: this.selectedServ,
    };
    try {
      Http.post('/getDataConfigPadraoTpCard',
        { requestType: 'Row', row: parametrosDaQuery }).then((response: IDictionary) => {
        const configCadastrada: IDictionary = response.data.dataset.result
          ? response.data.dataset.result
          : [];
        const cdservfili = configCadastrada[0]?.CDSERVFILI;
        if (configCadastrada.length > 0) {
          DialogService.show({
            name: 'casoPossuaConfig',
            type: 'warning',
            title: 'pme::atencao',
            text: 'pme::desejaImportarConfigPadrao',
            buttons: [
              {
                name: 'yes',
                component: 'ZdButton',
                label: 'pme::sim',
                events: { click: () => this.realizaImportacaoConfig(cdservfili) },
                outline: true,
              },
              {
                name: 'no',
                component: 'ZdButton',
                label: 'pme::nao',
                events: { click: () => DialogService.hide() },
              },
            ],
          });
        } else if (showMessage) {
          DialogService.show({
            name: 'casoNaoPossuaConfig',
            type: 'warning',
            title: 'pme::atencao',
            text: 'pme::nenhumConfigPadrao',
            buttons: [
              {
                name: 'ok',
                component: 'ZdButton',
                label: 'pme::OK',
                events: { click: () => DialogService.hide() },
                outline: true,
              },
            ],
          });
        }
      });
    } catch (Error) {
      this.utilities.showAlert('alertError', 'pme::erroImportacaoConfiguracao', 'error');
    }
  }

  public closeConfig() {
    const tabs = Metadata.getInstance<ITabs>('tabsConfigServ');
    
    if (tabs.activeTab === 0) {
      const configTipoPrat: GridEditable = Metadata.getInstance('CONFIGTIPOPRAT');
      this.gridEditController.cancelEditing({ component: configTipoPrat });
    } else {
      const custoPadr: GridEditable = Metadata.getInstance('CUSTPADR');
      this.gridEditController.cancelEditing({ component: custoPadr });
    }
    
    this.showTipoPrat = false;
    this.showConfig = false;
    this.colsServ = 12;
  }
  
  public closeCardConfig() {
    this.closeConfig();
    this.showCardConfig = false;
  }
  
  private onClickTipoDeCardPorUnid() {
    const cssClassName = 'gridTipoCardUnidade';
    const element = document.getElementsByClassName(cssClassName)[0];
    
    setTimeout(() => {
      // window.scrollTo(0, document.body.scrollHeight);
      element.scrollIntoView();
    }, 250);
  }
  
  private async createModalRelatorio() {
    this.modalRelatorio = await ModalService.createFromJson('pme/parametrizacao/modals/pme00301_relatorio_tipocard', true);
  }
  
  public openModalRelatorio() {
    if (!this.modalRelatorio) {
      this.createModalRelatorio().then(() => {
        this.modalRelatorio.show();
      });
    } else {
      this.modalRelatorio.show();
    }
  }
  
  public cancelModalRelatorio() {
    this.modalRelatorio.hide();
  }
  
  public clearFields({ component }: IEventParam<SelectMultiple>) {
    const servico = Metadata.getInstance<SelectMultiple>('SERVICO');
    const unidade = Metadata.getInstance<SelectMultiple>('UNIDADE');

    if (component.name === 'TIPOCARD_RELATORIO') {
      servico.setValue('');
    } else {
      const tipoCard = Metadata.getInstance<SelectMultiple>('TIPOCARD_RELATORIO');
      servico.setValue('');
      unidade.setValue('');
      tipoCard.setValue('');
    }
  }
  
  public setFilterServ({ component }: IEventParam<SelectMultiple>) {
    const tipoCard = Metadata.getInstance<SelectMultiple>('TIPOCARD_RELATORIO');
    let tpCardValue = '';
    if (tipoCard.returnObject) tpCardValue = tipoCard.value ? tipoCard.value.CDTIPOCARD : '';
    else tpCardValue = tipoCard.value ? tipoCard.value : '';

    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: tpCardValue.toString(),
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterAgreg({ component }: IEventParam<SelectMultiple>) {
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public setReportFieldsVisibility({ component }: IDictionary) {
    const relatorio = component.selectValue.CD_REPORT_CODE;
    this.clearFields(component);
    if (relatorio === 'PME00300') {
      this.reportUnidadeVisibility = false;
      this.reportServiceVisibility = true;
    } else {
      this.reportUnidadeVisibility = true;
      this.reportServiceVisibility = false;
    }
  }
  
  public clickButtonRelatorio({ component, event }: IDictionary) {
    const collapseCard: CollapseCard = component;
    const iconClicked = (event as IDictionary).target.classList[3]
      ? (event as IDictionary).target.classList[3]
      : (event as IDictionary).target.classList[0]; // classe utilizada como flag
    
    if (collapseCard.name === 'collapseCardTpCard' && collapseCard.expanded === false) {
      this.buttonRelatorioTpCard = true;
    } else if (collapseCard.name === 'collapseCardTpCard' && collapseCard.expanded) {
      if (iconClicked === 'mdi-file-document' || iconClicked === 'v-btn--round'
          || iconClicked === 'v-btn__content'
      ) {
        collapseCard.expanded = false;
      } else this.buttonRelatorioTpCard = false;
    }
    
    if (collapseCard.name === 'collapseCard' && collapseCard.expanded === false) {
      this.buttonRelatorioCollapse = true;
      this.onClickTipoDeCardPorUnid();
    } else if (collapseCard.name === 'collapseCard' && collapseCard.expanded) {
      if (iconClicked === 'mdi-file-document' || iconClicked === 'v-btn--round'
          || iconClicked === 'v-btn__content'
      ) {
        collapseCard.expanded = false;
      } else this.buttonRelatorioCollapse = false;
    }
  }
  
  private setParams(row: IDictionary) {
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
  }
  
  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORMRELTPCARDAPIO');

    const row: IDictionary = form.value;
    
    this.setParams(row);
    if (this.utilities.centralIsValid() && form.validate()) {
      this.reportController.callGenerateReport(component, row, 'report/tipoDeCardapio');
    }
  }
  
  public clickTipoCard({ component }: IEventParam<GridEditable>) {
    const gridTpCardIsEditing = component.editing;
    if (!gridTpCardIsEditing) {
      const element = document.getElementsByClassName('gridServico')[0];
      setTimeout(() => {
        element.scrollIntoView();
      }, 1);
    }
  }
  
  public cancelEditingTreeGrid() {
    const treeGrid: TreeGridEditable = Metadata.getInstance('TREEGRID');
    treeGrid.cancelEditedRows();
    const toolbarSlot = treeGrid.toolbarSlot;
    toolbarSlot.forEach((e: IComponentRender) => {
      if (e.name === 'cancelButton') e.disabled = !e.disabled;
      if (e.name === 'saveButton') e.disabled = !e.disabled;
    });
    treeGrid.datasource.get();
  }
  
  public updateTree() {
    const treeGrid = Metadata.getInstance<TreeGridEditable>('TREEGRID');
    const rowsEditadas = treeGrid.getEditedRows();
    const existemRowsEditadas = rowsEditadas.length !== 0;
    
    if (existemRowsEditadas) {
      rowsEditadas.forEach((row: IDictionary) => {
        row.CDFILCENTPME = this.appController.getCdCentral();
      });
      LoadingService.show(' ');
      Http.post('/updateTree', { requestType: 'Row', rowsEditadas }).then((response) => {
        if (response) this.utilities.showAlert('alert', 'pme::atualizadoComSucesso', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alertError', `pme::${error}`, 'error');
      }).finally(() => {
        LoadingService.hide();
        this.cancelEditingTreeGrid();
      });
    }
  }
  
  public valorCustoMask = {
    decimalPlaces: 2,
    decimalCharacter: (Config.masks.currencyMask as PmeCurrencyMask).decimalCharacter,
    digitGroupSeparator: (Config.masks.currencyMask as PmeCurrencyMask).digitGroupSeparator,
    currencySymbol: (Config.masks.currencyMask as PmeCurrencyMask).currencySymbol,
  };
}
